const { Expression, PartialEvalRewrite } = C.expr;

exports.name = 'Script';
exports.version = '0.1';
exports.disabled = false;
exports.destroyable = false;

const os = require('os');
const { spawn } = require('child_process');

const host = os.hostname();

let conf;
let batchSize;
let filter;
const requiredFields = ['host', 'source'];

exports.init = async (opts) => {
  conf = opts.conf;
  batchSize = conf.maxBatchSize || 10;
  filter = conf.filter || 'true';
  //TODO: do some arg validation here ...
  return Promise.resolve();
};

async function runScript(shellCmd, script) {
  return new Promise((resolve) => {
    const proc = spawn(shellCmd);
    let stdout = '';
    let stderr = '';
    proc.stdin.end(script);
    proc.stdout.on('data', (data) => stdout += data);
    proc.stderr.on('data', (data) => stderr += data);
    proc.on('close', (code) => resolve({code, stderr, stdout}))
  })
}

exports.discover = async (job) => {
  const filterExpr = new Expression(filter, { disallowAssign: true,
    partialEval: new PartialEvalRewrite((field) => !requiredFields.includes(field))
  });

  const cmdResult = await runScript(conf.shell || '/bin/bash', conf.discoverScript)

  if(cmdResult.code != 0) {
    job.logger().error('discover script failed', {exitCode: cmdResult.code, stderr: cmdResult.stderr});
    throw new Error(`Discover script failed, exitCode=${cmdResult.code}`);
  }

  let results = [];
  const lines = cmdResult.stdout.split('\n');
  for(let i=0; i<lines.length; i++) {
    const source = lines[i].trim();
    if(!source.length) continue; // ignore empty line
    const entry = { host, source };
    if (!filterExpr.evalOn(entry)) continue;
    results.push(entry);
    if(results.length >= batchSize) {
      await job.addResults(results);
      results.length = 0;
    }
  }
  if(results.length > 0) {
    await job.addResults(results);
  }
};

exports.collect = async (collectible, job) => {
  const env = {...process.env, CRIBL_COLLECT_ARG: collectible.source};
  job.logger().debug('starting collect script', {source: collectible.source});
  const proc = spawn(conf.shell || '/bin/bash', {env});
  proc.stdin.end(conf.collectScript);
  proc.on('exit', code => {
    if(code) {
      const msg = 'collect task failed';
      job.reportError(new Error(`${msg}, exitCode=${code}`)).catch(() => {});
      job.logger().error(msg, {exitCode: code, source: collectible.source});
    }
    else job.logger().info('collect task completed', {exitCode: code, source: collectible.source});
  });
  return Promise.resolve(proc.stdout);
};
